/* demo3g.c : demos de EZ-Draw reecrits en gtk+ 2.x
 *
 * Edouard.Thiel@lif.univ-mrs.fr - 23/03/2010 - version 0.9
 *
 * Compilation sous Unix :
 *     gcc -Wall demo3g.c ez-gtk.c -o demo3g `pkg-config gtk+-2.0 --cflags --libs`
 *
 * This program is free software under the terms of the
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

#include "ez-gtk.h"


/*
 * Appele'e par gtk_main lorsqu'il faut tout redessiner.
*/

gint area1_onExpose (GtkWidget *widget, GdkEventExpose *ev)
{
    int i, 
        w = widget->allocation.width, 
        h = widget->allocation.height;

    GdkGC *gc = ezg_get_gc (widget);

    /* On cree un layout pour dessiner du texte */
    PangoLayout *layout = gtk_widget_create_pango_layout (widget, NULL);

    /* On dessine le fond en blanc */
    ezg_area_clear (widget);

    ezg_set_color (gc, EZG_BLACK);
    for (i = 0; i <= 3; i++) {
        ezg_set_nfont (layout, i);
        ezg_draw_text (widget, gc, layout, EZG_MC, w/2, h/2 + 25*(i-2), 
            "Fonte numero %d", i);                         /* comme un printf */
    }

    ezg_set_color (gc, EZG_RED);
    ezg_set_nfont (layout, 0);
    ezg_draw_text (widget, gc, layout, EZG_TL,   2,   2, "Top\nLeft");
    ezg_draw_text (widget, gc, layout, EZG_TC, w/2,   2, "Top\nCenter");
    ezg_draw_text (widget, gc, layout, EZG_TR, w-2,   2, "Top\nRight");
    ezg_draw_text (widget, gc, layout, EZG_ML,   2, h/2, "Middle\nLeft");
    ezg_draw_text (widget, gc, layout, EZG_MR, w-2, h/2, "Middle\nRight");
    ezg_draw_text (widget, gc, layout, EZG_BL,   2, h-2, "Bottom\nLeft");
    ezg_draw_text (widget, gc, layout, EZG_BC, w/2, h-2, "Bottom\nCenter");
    ezg_draw_text (widget, gc, layout, EZG_BR, w-2, h-2, "Bottom\nRight");

    /* Libere la memoire */
    g_object_unref (G_OBJECT (layout));

    return TRUE;  /* L'evenement a ete traite' */
}


/*
 * Appele'e par gtk_main lorsqu'une touche a ete presse'e.
*/

gint area1_onKeyPress (GtkWidget *widget, GdkEventKey *ev)
{
    /* ev->keyval contient le symbole de la touche pressee */
    switch (ev->keyval) {
        case GDK_q : gtk_main_quit (); break;  /* touche 'q' */
    }

    return TRUE;  /* L'evenement a ete traite' */
}


int main (int argc, char *argv[])
{
    GtkWidget *win1, *area1;

    /* Initialise gtk et analyse la ligne de commande */
    gtk_init (&argc, &argv);
 
    /* Cree une fenetre, donne un titre et une taille */
    win1 = ezg_window_create (400, 300, "Demo 3g : affichage de texte");

    /* La destruction de la fenetre fera sortir de gtk_main */
    g_signal_connect (G_OBJECT (win1), "destroy", gtk_main_quit, NULL);

    /* On cree une zone de dessin, qui prend toute la place dans la fenetre */
    area1 = ezg_area_create (win1);

    /* On autorise certains evenements */
    GTK_WIDGET_SET_FLAGS  (area1, GTK_CAN_FOCUS);
    gtk_widget_set_events (area1, GDK_EXPOSURE_MASK | GDK_KEY_PRESS_MASK);

    /* On attache des callbacks pour gerer certains evenements */
    g_signal_connect (G_OBJECT (area1), "expose_event"   , G_CALLBACK (area1_onExpose)  , NULL);
    g_signal_connect (G_OBJECT (area1), "key_press_event", G_CALLBACK (area1_onKeyPress), NULL);

    /* Rend visibles la fenetre et son contenu */
    gtk_widget_show_all (win1);

    /* Boucle d'evenements qui fait "vivre" la fenetre */
    gtk_main ();

    exit (0);
}


