/* demo7g.c : demos de EZ-Draw reecrits en gtk+ 2.x
 *
 * Edouard.Thiel@lif.univ-mrs.fr - 13/05/2009 - version 0.9
 *
 * Compilation sous Unix :
 *     gcc -Wall demo7g.c ez-gtk.c -o demo7g `pkg-config gtk+-2.0 --cflags --libs`
 *
 * This program is free software under the terms of the
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

#include "ez-gtk.h"


#define MAX_TEXTE 80
char buf1[MAX_TEXTE] = "";


void affi_texte (GtkWidget *widget, GdkGC *gc, int x, int y, char *s)
{
    ezg_set_color (gc, EZG_BLACK);
    ezg_draw_text (widget, gc, NULL, EZG_TL, x, y, "Texte : %s_", s);
}


int saisie_texte (GtkWidget *widget, GdkEventKey *ev, char *s)
{
    int i;

    switch (ev->keyval) {

        case GDK_BackSpace :                              /* Touche backspace */
            i = strlen (s);
            if (i == 0) break;
            s[i-1] = 0;
            ezg_area_expose (widget);
            break;

        case GDK_Return : return 1;             /* Renvoie 1 si touche Entree */

        default :                                 /* Insertion d'un caractere */
            if (ev->length != 1) break;
            i = strlen (s);
            if (i >= MAX_TEXTE-1) break;
            s[i] = ev->string[0]; s[i+1] = 0;
            ezg_area_expose (widget);
            break;
    }
    return 0;
}


/*
 * Appele'e par gtk_main lorsqu'il faut tout redessiner.
*/

gint area1_onExpose (GtkWidget *widget, GdkEventExpose *ev)
{
    GdkGC *gc = ezg_get_gc (widget);

    ezg_area_clear (widget);

    affi_texte (widget, gc, 10, 10, buf1);

    return TRUE;  /* L'evenement a ete traite' */
}


/*
 * Appele'e par gtk_main lorsqu'une touche a ete presse'e.
*/

gint area1_onKeyPress (GtkWidget *widget, GdkEventKey *ev)
{
    if (saisie_texte (widget, ev, buf1) == 1) {

        GdkGC *gc = ezg_get_gc (widget);

        ezg_set_color (gc, EZG_RED);
        ezg_draw_text (widget, gc, NULL, EZG_TC, 200, 70,
            "Vous avez valide le texte :\n%s", buf1);
    }

    return TRUE;  /* L'evenement a ete traite' */
}


int main (int argc, char *argv[])
{
    GtkWidget *win1, *area1;

    /* Initialise gtk et analyse la ligne de commande */
    gtk_init (&argc, &argv);
 
    /* Cree une fenetre, donne un titre et une taille */
    win1 = ezg_window_create (400, 200, "Demo 7g : saisie de texte");

    /* La destruction de la fenetre fera sortir de gtk_main */
    g_signal_connect (G_OBJECT (win1), "destroy", gtk_main_quit, NULL);

    /* On cree une zone de dessin, qui prend toute la place dans la fenetre */
    area1 = ezg_area_create (win1);

    /* On autorise certains evenements */
    GTK_WIDGET_SET_FLAGS  (area1, GTK_CAN_FOCUS);
    gtk_widget_set_events (area1, GDK_EXPOSURE_MASK | GDK_KEY_PRESS_MASK);

    /* On attache des callbacks pour gerer certains evenements */
    g_signal_connect (G_OBJECT (area1), "expose_event"   , G_CALLBACK (area1_onExpose)  , NULL);
    g_signal_connect (G_OBJECT (area1), "key_press_event", G_CALLBACK (area1_onKeyPress), NULL);

    /* Rend visibles la fenetre et son contenu */
    gtk_widget_show_all (win1);

    /* Boucle d'evenements qui fait "vivre" la fenetre */
    gtk_main ();

    exit (0);
}

